jQuery(document).ready(function ($) {
    $(document).ajaxStart(function () {
        Pace.restart();
    });

    window.vueData.disableButtons = false;
    var skip = [];
    var app = new Vue({
        el: '#ezOptimize',
        data: window.vueData,
        methods: {
            checkApiKey: function () {
                if (this.apikey.length !== 0) {
                    return true;
                }

                swal({
                    type: 'error',
                    title: 'Fill in your API key first',
                    html: 'Visit your profile at <a href="https://ezoptimize.com/profile">EZOptimize.com</a> to obtain your API key'
                });
                return false;
            },
            process: function (itemPath) {
                var self = this,
                    dfd = new $.Deferred();
                $.post(ajaxurl, {
                    'action': 'ez_optimize',
                    'apikey': self.apikey,
                    'path': itemPath,
                }, function (response) {
                }, 'json')
                    .done(function (response) {
                        dfd.resolve(response);
                    })
                    .fail(function (jqXHR, textStatus, errorThrown) {
                        dfd.reject(jqXHR, textStatus, errorThrown);
                    });

                return dfd.promise();
            },
            optimizeSingle: function (e) {
                if (!this.checkApiKey()) {
                    return;
                }

                var self = this;

                // Process and wait for response
                this.process(e.currentTarget.getAttribute('data-path'))
                    .done(function (response) {
                        if (response.state === 'success') {
                            self.queue = response.vueData.queue;
                            self.optimized = response.vueData.optimized;
                        } else if (response.state === 'error') {
                            swal({
                                type: 'error',
                                title: 'Error',
                                html: response.message
                            });
                        }
                    })
                    .fail(function (data, textStatus, jqXHR) {
                        swal({
                            type: 'error',
                            title: 'Error',
                            html: `${jqXHR}.  Please contact EZOptimize Support!`
                        });
                    });
            },

            optimizeAll: function (e) {
                if (!this.checkApiKey() || this.queue.length < 1) {
                    return;
                }

                var self = this;

                var process = function (item) {
                    return new Promise(function (resolve, reject) {
                        // Process and wait for response
                        self.process(item.path)
                            .done(function (response) {
                                if (response.state === 'success') {
                                    self.queue = response.vueData.queue;
                                    self.optimized = response.vueData.optimized;
                                    resolve(true);
                                } else if (response.state === 'error') {
                                    self.queueError(item, response.message)
                                        .then(function (confirm) {
                                            if (confirm.value) {
                                                resolve();
                                            } else {
                                                reject();
                                            }
                                        });
                                }
                            })
                            .fail(function (data, textStatus, jqXHR) {
                                self.queueError(item, `${jqXHR}.  Please contact EZOptimize Support!`)
                                    .then(function (confirm) {
                                        if (confirm.value) {
                                            resolve();
                                        } else {
                                            reject();
                                        }
                                    });
                            });
                    });
                };

                var run = async function (queue) {
                    self.disableButtons = true;

                    // Enable navigation prompt
                    window.onbeforeunload = function () {
                        return true;
                    };

                    for (var index = 0; index < queue.length; index++) {
                        var check = await process(queue[index])
                            .then(function () {
                                return true;
                            })
                            .catch(function () {
                                return false;
                            });

                        if (!check) {
                            break;
                        }
                    }

                    self.disableButtons = false;

                    // Remove navigation prompt
                    window.onbeforeunload = null;
                };

                if (this.queue.length > 0) {
                    run(this.queue);
                }
            },
            queueError: function (item, message) {
                return swal({
                    title: "There was an error",
                    text: `The following image failed to process: ${item.name} - ${item.size.toFixed(1)}kb
                                    Error: ${message}`,
                    type: "warning",
                    showCancelButton: true,
                    confirmButtonColor: '#DD6B55',
                    confirmButtonText: 'Continue with the other images',
                    cancelButtonText: "Cancel"
                });
            },
            saveApiKey: function (e) {
                var data = {
                    'action': 'ez_api_key',
                    'apikey': this.apikey,
                    'auto': this.auto
                };

                // since 2.8 ajaxurl is always defined in the admin header and points to admin-ajax.php
                $.post(ajaxurl, data, function (response) {
                });
            },
            revert: function (e) {
                var self = this,
                    filePath = e.currentTarget.getAttribute('data-path');
                this.disableButtons = true;

                $.post(ajaxurl, {
                    'action': 'ez_revert',
                    'apikey': this.apikey,
                    'path': filePath
                }, function (response) {
                    self.disableButtons = false;
                    if (response.state === 'success') {
                        self.queue = response.queue;
                        self.optimized = response.optimized;
                    }
                }, 'json');
            }
        }
    });
});
